package net.wanji.opt.synthesis.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.DateStyle;
import net.wanji.common.framework.Constants;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.databus.dao.entity.GreenwaveInfoPO;
import net.wanji.databus.dao.mapper.BaseCrossInfoMapper;
import net.wanji.databus.dao.mapper.CrossDataRealtimeMapper;
import net.wanji.databus.dao.mapper.GreenwaveInfoMapper;
import net.wanji.databus.po.BaseCrossInfoPO;
import net.wanji.databus.vo.AbnormalCrossListVO;
import net.wanji.opt.cache.BaseCrossInfoCache;
import net.wanji.opt.cache.GreenWaveInfoCache;
import net.wanji.opt.common.enums.StrategyControlEnum;
import net.wanji.opt.dao.mapper.*;
import net.wanji.opt.po.StrategyGreenOptHistEntity;
import net.wanji.opt.synthesis.enums.StrategyCrossAlgoEnum;
import net.wanji.opt.synthesis.pojo.*;
import net.wanji.opt.synthesis.pojo.vo.StrategyControlDataVO;
import net.wanji.opt.synthesis.pojo.vo.StrategyControlHistVO;
import net.wanji.opt.synthesis.pojo.vo.StrategyControlVO;
import net.wanji.opt.synthesis.pojo.vo.StrategyOptTimesVO;
import net.wanji.opt.synthesis.service.PushStrategyControlService;
import net.wanji.opt.synthesis.service.StrategyControlService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.lang.reflect.InvocationTargetException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2024/11/03 17:37
 */
@Slf4j
@Service
public class StrategyControlServiceImpl implements StrategyControlService {

    @Resource
    private PushStrategyControlService pushStrategyControlService;
    @Resource
    private StrategyControlInfoMapper strategyControlInfoMapper;
    @Resource
    private StrategyPlanInfoMapper strategyPlanInfoMapper;
    @Resource
    private StrategyDailyPlanInfoMapper strategyDailyPlanInfoMapper;
    @Resource
    private BaseCrossInfoMapper baseCrossInfoMapper;
    @Resource
    private StrategyFactoryMapper strategyFactoryMapper;
    @Resource
    private GreenwaveInfoMapper greenwaveInfoMapper;
    @Resource
    private StrategyGreenOptHistMapper strategyGreenOptHistMapper;
    @Resource
    private StrategyCrossResultMapper strategyCrossResultMapper;
    @Resource
    private BaseCrossInfoCache baseCrossInfoCache;
    @Resource
    private CrossDataRealtimeMapper crossDataRealtimeMapper;


    @Override
    public JsonViewObject strategyInfoOperation(StrategyControlVO strategyControlVO) throws Exception {

        try {
            // 存库
            insertAndUpdate(strategyControlVO);

            StrategyControlReq req = convertReq(strategyControlVO);
            Result result = pushStrategyControlService.push(req);
            if (Objects.nonNull(result)) {
                JsonViewObject jsonViewObject = JsonViewObject.newInstance();
                jsonViewObject.setCode(Integer.valueOf(result.getCode()));
                jsonViewObject.setMessage(result.getMsg());
                jsonViewObject.setContent(result.getData());
                jsonViewObject.setStatus(StringUtils.equals("200", result.getCode()) ? Constants.JsonView.STATUS_SUCCESS : Constants.JsonView.STATUS_FAIL);
                return jsonViewObject;
            } else {
                return JsonViewObject.newInstance().fail("推送神思策略控制失败！");
            }
        } catch (Exception e) {
            log.error("策略操作失败：{}", e);
            throw new Exception(e);
        }
    }

    private void insertAndUpdate(StrategyControlVO strategyControlVO) throws IllegalAccessException, InvocationTargetException, JsonProcessingException {
        List<StrategyControlDataVO> dataList = strategyControlVO.getDataList();
        for (StrategyControlDataVO dataVO : dataList) {
            StrategyControlDataEntity entity = new StrategyControlDataEntity();
            BeanUtils.copyProperties(dataVO, entity);
            entity.setTime(JacksonUtils.getInstance().writeValueAsString(dataVO.getTime()));
            if (StringUtils.equals("insert", dataVO.getAction())) {
                entity.setId(null);
                strategyControlInfoMapper.insert(entity);
            } else if (StringUtils.equals("update", dataVO.getAction())) {
                strategyControlInfoMapper.updateById(entity);
            } else if (StringUtils.equals("delete", dataVO.getAction())) {
                strategyControlInfoMapper.deleteById(entity);
            }
        }
    }

    private StrategyControlReq convertReq(StrategyControlVO vo) throws Exception {
        try {
            List<StrategyControlDataVO> dataList = vo.getDataList();
            StrategyControlReq req = new StrategyControlReq();
            if (!CollectionUtils.isEmpty(dataList)) {
                List<StrategyControlDataReq> dataReqList = new ArrayList<>(dataList.size());
                for (StrategyControlDataVO dataVO : dataList) {
                    StrategyControlDataReq detailData = new StrategyControlDataReq();
                    detailData.setBiz_id(dataVO.getBizId());
                    detailData.setBiz_type(dataVO.getBizType());
                    detailData.setStrategy(dataVO.getStrategy());
                    detailData.setSchedule_start(DateUtil.format(dataVO.getScheduleStart(), DateStyle.YYYY_MM_DD_HH_MM_SS.getValue()));
                    detailData.setSchedule_end(DateUtil.format(dataVO.getScheduleEnd(), DateStyle.YYYY_MM_DD_HH_MM_SS.getValue()));
                    List<StrategyControlDataVO.TimeTable> timeTableList = dataVO.getTime();
                    List<StrategyControlDataReq.Time_table> timeTables = new ArrayList<>(timeTableList.size());
                    timeTableList.forEach(item -> {
                        StrategyControlDataReq.Time_table timeTable = new StrategyControlDataReq.Time_table();
                        timeTable.setTime_list(item.getTimeList());
                        timeTable.setWeek(item.getWeek());
                        timeTables.add(timeTable);
                    });
                    detailData.setTime_table(timeTables);
                    detailData.setFreq(dataVO.getFrequency());
                    detailData.setStatus(dataVO.getStatus());
                    detailData.setAction(StringUtils.equals("update", dataVO.getAction()) ? "insert" : dataVO.getAction());

                    dataReqList.add(detailData);
                    req.setData(dataReqList);
                }
            }
            return req;
        } catch (Exception e) {
            log.error("策略控制操作数据转换异常：{}", e);
            throw new Exception(e);
        }
    }

    @Override
    public JsonViewObject strategyInfoPageList(StrategyControlDataEntity entity) throws Exception {
        LambdaQueryWrapper<StrategyControlDataEntity> queryWrapper = new LambdaQueryWrapper<>();
        if (StringUtils.isNotBlank(entity.getBizId())) {
            queryWrapper.eq(StrategyControlDataEntity::getBizId, entity.getBizId());
        }
        if (Objects.nonNull(entity.getBizType())) {
            queryWrapper.eq(StrategyControlDataEntity::getBizType, entity.getBizType());
        }
        if (Objects.nonNull(entity.getStrategy())) {
            queryWrapper.eq(StrategyControlDataEntity::getStrategy, entity.getStrategy());
        }
        if (Objects.nonNull(entity.getStatus())) {
            queryWrapper.eq(StrategyControlDataEntity::getStatus, entity.getStatus());
        }
        Page<StrategyControlDataEntity> page = new Page<>(entity.getPageNum(), entity.getPageSize());
        Page<StrategyControlDataEntity> pageInfo = strategyControlInfoMapper.selectPage(page, queryWrapper);
        return JsonViewObject.newInstance().success(pageInfo.getRecords());
    }

    @Override
    public JsonViewObject crossStrategyHistList() throws Exception {
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        List<StrategyControlHistVO> results = new ArrayList<>();
        LocalDate currentDate = LocalDate.now();
        LocalDateTime midnight = currentDate.atStartOfDay();
        // 查询当前绿波历史记录
        setGreenOptHist(results, midnight, format);
        setCrossOptHist(results, midnight);
        List<StrategyControlHistVO> sorted = results.stream().sorted(Comparator.comparing(StrategyControlHistVO::getOptTime).reversed()).collect(Collectors.toList());
        return JsonViewObject.newInstance().success(sorted);
    }

    private void setCrossOptHist(List<StrategyControlHistVO> results, LocalDateTime midnight) {
        LambdaQueryWrapper<StrategyCrossResultEntity> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.ge(StrategyCrossResultEntity::getIssueTime, midnight);
        List<StrategyCrossResultEntity> resultEntities = strategyCrossResultMapper.selectList(queryWrapper);
        if (!CollectionUtils.isEmpty(resultEntities)) {
            for (StrategyCrossResultEntity resultEntity : resultEntities) {
                StrategyControlHistVO histVO = new StrategyControlHistVO();
                String crossId = resultEntity.getCrossId();
                histVO.setId(crossId);
                histVO.setName(resultEntity.getCrossName());
                String coordinateByCrossId = BaseCrossInfoCache.getCoordinateByCrossId(crossId);
                histVO.setWkt(coordinateByCrossId.replace("POINT(", "").replace(" ", ",").replace(")", ""));
                histVO.setStrategy(resultEntity.getCurrentAlgo());
                histVO.setStrategyName(StrategyCrossAlgoEnum.getDescByCode(resultEntity.getCurrentAlgo()));
                histVO.setOptTime(resultEntity.getIssueTime());
                histVO.setResult(Objects.equals(200, resultEntity.getResponseCode()) ? "成功" : "失败");
                results.add(histVO);
            }
        }
    }

    private void setGreenOptHist(List<StrategyControlHistVO> results, LocalDateTime midnight, SimpleDateFormat format) throws ParseException {
        LambdaQueryWrapper<StrategyGreenOptHistEntity> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.ge(StrategyGreenOptHistEntity::getControlTime, midnight);
        List<StrategyGreenOptHistEntity> entities = strategyGreenOptHistMapper.selectList(queryWrapper);
        if (!CollectionUtils.isEmpty(entities)) {
            Map<Integer, GreenwaveInfoPO> greenWaveMap = GreenWaveInfoCache.greenWaveMap;
            for (StrategyGreenOptHistEntity entity : entities) {
                StrategyControlHistVO histVO = new StrategyControlHistVO();
                histVO.setId(String.valueOf(entity.getGreenId()));
                if (greenWaveMap.isEmpty()) {
                    continue;
                }
                String controlTime = entity.getControlTime();
                Date parse = format.parse(controlTime);
                GreenwaveInfoPO greenwaveInfoPO = greenWaveMap.get(entity.getGreenId());
                if (Objects.nonNull(greenwaveInfoPO)) {
                    histVO.setName(greenwaveInfoPO.getName());
                    histVO.setWkt(greenwaveInfoPO.getWkt());
                }
                LambdaQueryWrapper<StrategyControlDataEntity> strategyQuery = new LambdaQueryWrapper<>();
                strategyQuery.eq(StrategyControlDataEntity::getBizId, entity.getGreenId());
                strategyQuery.eq(StrategyControlDataEntity::getBizType, 1);
                strategyQuery.le(StrategyControlDataEntity::getScheduleStart, parse);
                strategyQuery.ge(StrategyControlDataEntity::getScheduleEnd, parse);
                // 应该是下发时间在调度下发时间内，但是目前神思下发可能没有走策略
                List<StrategyControlDataEntity> strategyList = strategyControlInfoMapper.selectList(strategyQuery);
                if (!CollectionUtils.isEmpty(strategyList)) {
                    StrategyControlDataEntity strategyControlDataEntity = strategyList.get(0);
                    histVO.setStrategy(strategyControlDataEntity.getStrategy());
                    histVO.setStrategyName("绿波带");
                }
                histVO.setOptTime(entity.getControlTime());
                histVO.setResult(Objects.equals(1, entity.getControlMethod()) ? "失败" : "成功");
                results.add(histVO);
            }
        }
    }

    @Override
    public JsonViewObject crossStrategyInfoList(Integer type) throws Exception {
        try {
            List<StrategyControlDataEntity> results = new ArrayList<>();
            List<StrategyControlDataEntity> entities = getCurrentStrateInfoList(type);
            List<StrategyControlDataEntity> runList = entities.stream().filter(entity -> Objects.equals(1, entity.getStatus())).collect(Collectors.toList());
            Map<String, StrategyControlDataEntity> map = runList.stream().collect(Collectors.toMap(StrategyControlDataEntity::getBizId, entity -> entity));
            // 路口
            if (Objects.equals(0, type)) {
                List<AbnormalCrossListVO> abnormalCrossListVOS = crossDataRealtimeMapper.selectAbnormalCross(null, null, null);
                if (!CollectionUtils.isEmpty(abnormalCrossListVOS)) {
                    for (AbnormalCrossListVO baseCrossInfoPO : abnormalCrossListVOS) {
                        StrategyControlDataEntity strategyControlDataEntity = new StrategyControlDataEntity();
                        String crossId = baseCrossInfoPO.getId();
                        strategyControlDataEntity.setBizType(0);
                        strategyControlDataEntity.setBizId(crossId);
                        // 转化成 0 畅通 1失衡 2拥堵 3溢出 4死锁 5空放 6干线拥堵 7干线缓行 8干线畅通
                        Integer status = baseCrossInfoPO.getRealtimeStatus();
                        strategyControlDataEntity.setStrategy(status);
                        strategyControlDataEntity.setOptMethod(Objects.equals(1, status) ? "均衡调控" : "效率提升");
                        strategyControlDataEntity.setTime(null);
                        strategyControlDataEntity.setStatus(status);
                        strategyControlDataEntity.setCrossName(baseCrossInfoPO.getName());
                        String location = baseCrossInfoPO.getLocationStr();
                        location = location.replace("POINT(", "").replace(" ", ",").replace(")", "");
                        strategyControlDataEntity.setWkt(location);
                        results.add(strategyControlDataEntity);
                    }
                }
            } // 干线
            if (Objects.equals(1, type)) {
                List<GreenwaveInfoPO> greenwaveInfoPOS = greenwaveInfoMapper.selectAll();
                if (!CollectionUtils.isEmpty(greenwaveInfoPOS)) {
                    for (GreenwaveInfoPO greenwaveInfoPO : greenwaveInfoPOS) {
                        StrategyControlDataEntity strategyControlDataEntity = new StrategyControlDataEntity();
                        String greenId = String.valueOf(greenwaveInfoPO.getId());
                        strategyControlDataEntity.setBizType(1);
                        strategyControlDataEntity.setBizId(String.valueOf(greenId));
                        strategyControlDataEntity.setStrategy(8);
                        strategyControlDataEntity.setTime(null);
                        strategyControlDataEntity.setStatus(0);
                        if (map.containsKey(greenId)) {
                            strategyControlDataEntity = map.get(greenId);
                        }
                        strategyControlDataEntity.setCrossName(greenwaveInfoPO.getName());
                        strategyControlDataEntity.setWkt(greenwaveInfoPO.getWkt());
                        results.add(strategyControlDataEntity);
                    }
                }
            }
            return JsonViewObject.newInstance().success(results);
        } catch (Exception e) {
            log.error("策略控制查询列表：{}", e);
            throw new RuntimeException(e);
        }
    }

    @Override
    public JsonViewObject strategyOptTimes() throws Exception {
        LocalDate currentDate = LocalDate.now();
        LocalTime startTime = LocalTime.MIDNIGHT;
        LocalDateTime startOfDay = LocalDateTime.of(currentDate, startTime);
        LambdaQueryWrapper<StrategyGreenOptHistEntity> greenQuery = new LambdaQueryWrapper<>();
        greenQuery.ge(StrategyGreenOptHistEntity::getControlTime, startOfDay);
        List<StrategyGreenOptHistEntity> greenOptList = strategyGreenOptHistMapper.selectList(greenQuery);
        int totalTime = 0;
        int totalCount = 0;
        if (!CollectionUtils.isEmpty(greenOptList)) {
            for (StrategyGreenOptHistEntity entity : greenOptList) {
                Integer controlDuration = entity.getControlDuration();
                totalTime += controlDuration;
                totalCount += 1;
            }
        }
        LambdaQueryWrapper<StrategyCrossResultEntity> crossQuery = new LambdaQueryWrapper<>();
        crossQuery.ge(StrategyCrossResultEntity::getIssueTime, startOfDay);
        List<StrategyCrossResultEntity> crossList = strategyCrossResultMapper.selectList(crossQuery);
        if (!CollectionUtils.isEmpty(crossList)) {
            for (StrategyCrossResultEntity entity : crossList) {
                Integer countDown = entity.getCountDown();
                if (Math.abs(countDown) > 1) {
                    totalTime += countDown;
                    totalCount += 1;
                }
            }
        }
        StrategyOptTimesVO strategyOptTimesVO = new StrategyOptTimesVO();
        strategyOptTimesVO.setCount(totalCount);
        strategyOptTimesVO.setTimes(totalTime / 3600);
        return JsonViewObject.newInstance().success(strategyOptTimesVO);
    }

    private List<StrategyControlDataEntity> getCurrentStrateInfoList(Integer type) throws Exception {
        ObjectMapper instance = JacksonUtils.getInstance();
        LambdaQueryWrapper<StrategyControlDataEntity> queryWrapper = new LambdaQueryWrapper<>();
        Date current = new Date();
        queryWrapper.eq(StrategyControlDataEntity::getBizType, type);
        queryWrapper.le(StrategyControlDataEntity::getScheduleStart, current);
        queryWrapper.ge(StrategyControlDataEntity::getScheduleEnd, current);
        //queryWrapper.ge(StrategyControlDataEntity::getScheduleStart, current);
        //queryWrapper.le(StrategyControlDataEntity::getScheduleEnd, current);
        List<StrategyControlDataEntity> entities = strategyControlInfoMapper.selectList(queryWrapper);
        List<StrategyControlDataEntity> results = new ArrayList<>(entities.size());
        for (StrategyControlDataEntity entity : entities) {
            StrategyControlDataEntity result = new StrategyControlDataEntity();
            BeanUtils.copyProperties(entity, result);
            String time = entity.getTime();
            List<StrategyControlDataVO.TimeTable> timeTables = instance.readValue(time, new TypeReference<List<StrategyControlDataVO.TimeTable>>() {
            });
            for (StrategyControlDataVO.TimeTable timeTable : timeTables) {
                String[] timeList = timeTable.getTimeList();
                for (String s : timeList) {
                    String[] hours = s.split(",");
                    for (String hour : hours) {
                        String[] currentHour = hour.split("-");
                        String startHour = currentHour[0];
                        String entHour = currentHour[1];
                        if (StringUtils.equals("24:00", entHour)) {
                            entHour = "23:59";
                        }
                        String format = DateUtil.format(current, "HH:mm");
                        DateTime currentTime = DateUtil.parse(format, "HH:mm");
                        DateTime startHourDate = DateUtil.parse(startHour, "HH:mm");
                        DateTime endHourDate = DateUtil.parse(entHour, "HH:mm");
                        if (currentTime.isAfter(startHourDate) && currentTime.isBefore(endHourDate)) {
                            result.setTime(hour);
                            result.setStatus(1);
                        } else {
                            // 如果有调度策略在执行，设置为开启，否则关闭，提供前端过滤配置和未配置
                            result.setStatus(0);
                            result.setTime(null);
                        }
                    }
                }
            }
            if (type == 1) {
                Map<Integer, GreenwaveInfoPO> greenWaveMap = GreenWaveInfoCache.greenWaveMap;
                if (!greenWaveMap.isEmpty()) {
                    GreenwaveInfoPO greenwaveInfoPO = greenWaveMap.get(Integer.valueOf(entity.getBizId()));
                    result.setCrossName(greenwaveInfoPO.getName());
                    result.setWkt(greenwaveInfoPO.getWkt());
                }
            }
            result.setOptMethod(StrategyControlEnum.getMethod(result.getStrategy()));
            results.add(result);
        }
        return results;
    }

    @Override
    public JsonViewObject crossOptInfoList(Integer type) throws Exception {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        try {
            List<StrategyControlDataEntity> currentStrateInfoList = getCurrentStrateInfoList(type);
            List<StrategyControlDataExt> strategyControlDataExts = new ArrayList<>();
            List<StrategyControlDataExt> results = new ArrayList<>();
            for (StrategyControlDataEntity strategyControlDataEntity : currentStrateInfoList) {
                Integer strategy = strategyControlDataEntity.getStrategy();
                StrategyControlDataExt strategyControlDataExt = new StrategyControlDataExt();
                BeanUtils.copyProperties(strategyControlDataEntity, strategyControlDataExt);
                strategyControlDataExt.setStrategyName(StrategyControlEnum.getDesc(strategy));
                strategyControlDataExt.setOptMethod(StrategyControlEnum.getMethod(strategy));
                strategyControlDataExt.setOptStatus("未优化");
                if (StringUtils.isNotBlank(strategyControlDataEntity.getTime())) {
                    strategyControlDataExt.setOptStatus("优化中");
                    strategyControlDataExts.add(strategyControlDataExt);
                }
            }
            // 路口无优化模式
            if (Objects.equals(0, type)) {
                List<BaseCrossInfoPO> crossInfoCache = baseCrossInfoCache.getCrossInfoCache();
                if (!CollectionUtils.isEmpty(crossInfoCache)) {
                    for (BaseCrossInfoPO baseCrossInfoPO : crossInfoCache) {
                        StrategyControlDataExt ext = new StrategyControlDataExt();
                        ext.setBizId(baseCrossInfoPO.getId());
                        ext.setStrategyName("无策略");
                        ext.setOptStatus("未优化");
                        ext.setOptMethod("畅通");
                        ext.setStatus(0);
                        for (StrategyControlDataExt strategyControlDataExt : strategyControlDataExts) {
                            if (StringUtils.equals(baseCrossInfoPO.getId(), strategyControlDataExt.getBizId())) {
                                ext = strategyControlDataExt;
                            }
                        }
                        String location = baseCrossInfoPO.getLocation();
                        location = location.replace("POINT(", "").replace(" ", ",").replace(")", "");
                        ext.setWkt(location);
                        ext.setCrossName(baseCrossInfoPO.getName());
                        results.add(ext);
                    }
                }
            }
            // 干线无优化模式
            if (Objects.equals(1, type)) {
                if (CollectionUtils.isEmpty(strategyControlDataExts)) {
                    Map<Integer, GreenwaveInfoPO> greenWaveMap = GreenWaveInfoCache.greenWaveMap;
                    for (Map.Entry<Integer, GreenwaveInfoPO> entry : greenWaveMap.entrySet()) {
                        StrategyControlDataExt ext = new StrategyControlDataExt();
                        ext.setStrategy(0);
                        ext.setStrategyName("无策略");
                        ext.setOptStatus("未优化");
                        ext.setOptMethod("效率提升");
                        ext.setStatus(0);
                        ext.setWkt(entry.getValue().getWkt());
                        ext.setCrossName(entry.getValue().getName());
                        ext.setBizId(String.valueOf(entry.getKey()));
                        results.add(ext);
                    }
                } else {
                    results.addAll(strategyControlDataExts);
                }
            }
            results.sort(Comparator.comparing(StrategyControlDataExt::getOptStatus));
            return jsonViewObject.success(results, "路网优化监测查询成功");
        } catch (Exception e) {
            log.error("路网优化监测查询失败: {}", e);
            return jsonViewObject.fail("路网优化监测查询失败");
        }
    }

    @Override
    public JsonViewObject strategyPlanDetail(String crossId) throws Exception {
        try {
            ObjectMapper instance = JacksonUtils.getInstance();
            StrategyControlDetailList result = new StrategyControlDetailList();
            LambdaQueryWrapper<StrategyPlanInfoEntity> planQueryWrapper = new LambdaQueryWrapper<>();
            planQueryWrapper.eq(StrategyPlanInfoEntity::getCrossId, crossId);
            List<StrategyPlanInfoEntity> planInfoEntities = strategyPlanInfoMapper.selectList(planQueryWrapper);
            List<StrategyControlDetailList.ExecutePlan> executePlans = new ArrayList<>();
            for (StrategyPlanInfoEntity planInfoEntity : planInfoEntities) {
                StrategyControlDetailList.ExecutePlan executePlan = new StrategyControlDetailList.ExecutePlan();
                executePlan.setCompany(planInfoEntity.getCompany());
                executePlan.setCrossId(planInfoEntity.getCrossId());
                executePlan.setType(planInfoEntity.getType());
                executePlan.setPlanId(planInfoEntity.getPlanId());
                executePlan.setStartTime(planInfoEntity.getStartTime());
                executePlan.setEndTime(planInfoEntity.getEndTime());
                executePlan.setStatus(planInfoEntity.getStatus());
                executePlan.setType(planInfoEntity.getType());
                String planDetailsStr = planInfoEntity.getPlanDetails();
                List<StrategyControlDetailList.ExecutePlan.PlanDetail> planDetails = instance.readValue(planDetailsStr, new TypeReference<List<StrategyControlDetailList.ExecutePlan.PlanDetail>>() {
                });
                executePlan.setPlanDetails(planDetails);
                executePlans.add(executePlan);
            }
            result.setPlans(executePlans);

            LambdaQueryWrapper<StrategyDailyPlanInfoEntity> dailyPlanQueryWrapper = new LambdaQueryWrapper<>();
            dailyPlanQueryWrapper.eq(StrategyDailyPlanInfoEntity::getCrossId, crossId);
            List<StrategyDailyPlanInfoEntity> dailyPlanInfoEntities = strategyDailyPlanInfoMapper.selectList(dailyPlanQueryWrapper);
            List<StrategyControlDetailList.DailyPlan> dailyPlans = new ArrayList<>();
            for (StrategyDailyPlanInfoEntity dailyPlanInfoEntity : dailyPlanInfoEntities) {
                StrategyControlDetailList.DailyPlan dailyPlan = new StrategyControlDetailList.DailyPlan();
                dailyPlan.setCrossId(dailyPlanInfoEntity.getCrossId());
                dailyPlan.setDailyPlanId(dailyPlanInfoEntity.getDailyPlanId());
                String dailyPlanDetailsStr = dailyPlanInfoEntity.getDailyPlanDetails();
                List<StrategyControlDetailList.DailyPlan.DailyPlanDetail> dailyPlanDetails = instance.readValue(dailyPlanDetailsStr, new TypeReference<List<StrategyControlDetailList.DailyPlan.DailyPlanDetail>>() {
                });
                dailyPlan.setDailyPlanDetails(dailyPlanDetails);
                dailyPlans.add(dailyPlan);
            }
            result.setDailyPlans(dailyPlans);
            result.setCrossId(crossId);
            return JsonViewObject.newInstance().success(result);
        } catch (Exception e) {
            log.error("策略管理查询失败：{}", e);
            throw new Exception(e);
        }
    }

    @Override
    public JsonViewObject strategyPlanSave(StrategyControlDetailList list) throws Exception {
        try {
            if (Objects.nonNull(list)) {
                ObjectMapper mapper = JacksonUtils.getInstance();
                List<StrategyControlDetailList.ExecutePlan> plans = list.getPlans();
                if (!CollectionUtils.isEmpty(plans)) {
                    for (StrategyControlDetailList.ExecutePlan plan : plans) {
                        StrategyPlanInfoEntity planInfoEntity = new StrategyPlanInfoEntity();
                        planInfoEntity.setPlanId(plan.getPlanId());
                        planInfoEntity.setCrossId(plan.getCrossId());
                        planInfoEntity.setType(plan.getType());
                        planInfoEntity.setStartTime(plan.getStartTime());
                        planInfoEntity.setEndTime(plan.getEndTime());
                        planInfoEntity.setCompany(plan.getCompany());
                        planInfoEntity.setPlanDetails(mapper.writeValueAsString(plan.getPlanDetails()));
                        LambdaQueryWrapper<StrategyPlanInfoEntity> planDel = new LambdaQueryWrapper<>();
                        planDel.eq(StrategyPlanInfoEntity::getCrossId, plan.getCrossId());
                        planDel.eq(StrategyPlanInfoEntity::getPlanId, plan.getPlanId());
                        strategyPlanInfoMapper.delete(planDel);
                        strategyPlanInfoMapper.insert(planInfoEntity);
                    }
                }

                List<StrategyControlDetailList.DailyPlan> dailyPlans = list.getDailyPlans();
                for (StrategyControlDetailList.DailyPlan dailyPlan : dailyPlans) {
                    Integer dailyPlanId = dailyPlan.getDailyPlanId();
                    String crossId = dailyPlan.getCrossId();
                    List<StrategyControlDetailList.DailyPlan.DailyPlanDetail> dailyPlanDetails = dailyPlan.getDailyPlanDetails();
                    StrategyDailyPlanInfoEntity dailyPlanInfoEntity = new StrategyDailyPlanInfoEntity();
                    dailyPlanInfoEntity.setCrossId(crossId);
                    dailyPlanInfoEntity.setDailyPlanId(dailyPlanId);
                    LambdaQueryWrapper<StrategyDailyPlanInfoEntity> delete = new LambdaQueryWrapper<>();
                    delete.eq(StrategyDailyPlanInfoEntity::getCrossId, crossId);
                    delete.eq(StrategyDailyPlanInfoEntity::getDailyPlanId, dailyPlanId);
                    strategyDailyPlanInfoMapper.delete(delete);
                    dailyPlanInfoEntity.setDailyPlanDetails(mapper.writeValueAsString(dailyPlanDetails));
                    strategyDailyPlanInfoMapper.insert(dailyPlanInfoEntity);
                }
            }
            return JsonViewObject.newInstance().success();
        } catch (Exception e) {
            log.error("策略管理数据保存失败，：{}", e);
            throw new Exception(e);
        }
    }

    @Override
    public JsonViewObject strategyPush(StrategyControlDetailList list) throws Exception {
        List<StrategyControlDetailList.ExecutePlan> plans = list.getPlans();
        StrategyControlDetailList.ExecutePlan executePlan = plans.get(0);
        LambdaQueryWrapper<StrategyPlanInfoEntity> query = new LambdaQueryWrapper<>();
        query.eq(StrategyPlanInfoEntity::getPlanId, executePlan.getPlanId());
        query.eq(StrategyPlanInfoEntity::getCrossId, executePlan.getCrossId());
        strategyPlanInfoMapper.delete(query);
        StrategyPlanInfoEntity planInfoEntity = new StrategyPlanInfoEntity();
        planInfoEntity.setCrossId(executePlan.getCrossId());
        planInfoEntity.setPlanId(executePlan.getPlanId());
        planInfoEntity.setType(executePlan.getType());
        planInfoEntity.setCompany("A厂商");
        planInfoEntity.setStatus(executePlan.getStatus());
        planInfoEntity.setStartTime(executePlan.getStartTime());
        planInfoEntity.setEndTime(executePlan.getEndTime());
        planInfoEntity.setPlanDetails(JacksonUtils.getInstance().writeValueAsString(executePlan.getPlanDetails()));
        strategyPlanInfoMapper.insert(planInfoEntity);
        StrategyControlVO strategyControlVO = convertDataList(list);
        StrategyControlReq req = convertReq(strategyControlVO);

        List<StrategyControlDataReq> data = req.getData();
        for (StrategyControlDataReq datum : data) {
            StrategyControlDataEntity strategyControlDataEntity = new StrategyControlDataEntity();
            strategyControlDataEntity.setBizId(datum.getBiz_id());
            strategyControlDataEntity.setBizType(datum.getBiz_type());
            strategyControlDataEntity.setStrategy(datum.getStrategy());
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            strategyControlDataEntity.setScheduleStart(simpleDateFormat.parse(datum.getSchedule_start()));
            strategyControlDataEntity.setScheduleStart(simpleDateFormat.parse(datum.getSchedule_start()));
            ObjectMapper instance = JacksonUtils.getInstance();
            strategyControlDataEntity.setTime(instance.writeValueAsString(datum.getTime_table()));
            strategyControlDataEntity.setFrequency(datum.getStrategy());
            strategyControlDataEntity.setStatus(datum.getStatus());
            strategyControlInfoMapper.insert(strategyControlDataEntity);
        }

        log.error("下发神思策略信息详情： {}", JacksonUtils.getInstance().writeValueAsString(req));
        //Result result = pushStrategyControlService.push(req);
        Result result = new Result("200", "成功", null);
        if (Objects.nonNull(result)) {
            JsonViewObject jsonViewObject = JsonViewObject.newInstance();
            jsonViewObject.setCode(Integer.valueOf(result.getCode()));
            jsonViewObject.setMessage(result.getMsg());
            jsonViewObject.setContent(result.getData());
            jsonViewObject.setStatus(StringUtils.equals("200", result.getCode()) ? Constants.JsonView.STATUS_SUCCESS : Constants.JsonView.STATUS_FAIL);
            return jsonViewObject;
        } else {
            return JsonViewObject.newInstance().fail("推送神思策略控制失败！");
        }
    }

    /**
     * 页面管理数据转化神思数据
     *
     * @param list
     */
    public StrategyControlVO convertDataList(StrategyControlDetailList list) throws Exception {
        try {
            if (Objects.nonNull(list)) {
                List<StrategyControlDetailList.DailyPlan> dailyPlans = list.getDailyPlans();
                List<StrategyPlanDTO> strategyPlanDTOS = new ArrayList<>();
                if (!CollectionUtils.isEmpty(dailyPlans)) {
                    for (StrategyControlDetailList.DailyPlan dailyPlan : dailyPlans) {
                        Integer dailyPlanId = dailyPlan.getDailyPlanId();
                        String crossId = dailyPlan.getCrossId();
                        List<StrategyControlDetailList.DailyPlan.DailyPlanDetail> dailyPlanDetails = dailyPlan.getDailyPlanDetails();

                        List<StrategyControlDetailList.DailyPlan.DailyPlanDetail> dailyPlanGroupList = new ArrayList<>(dailyPlanDetails.size());
                        Map<Integer, List<StrategyControlDetailList.DailyPlan.DailyPlanDetail>> dailyMap = dailyPlanDetails.stream().collect(Collectors.groupingBy(StrategyControlDetailList.DailyPlan.DailyPlanDetail::getStrategy));
                        for (Map.Entry<Integer, List<StrategyControlDetailList.DailyPlan.DailyPlanDetail>> entry : dailyMap.entrySet()) {
                            StrategyControlDetailList.DailyPlan.DailyPlanDetail result = new StrategyControlDetailList.DailyPlan.DailyPlanDetail();
                            Integer strategy = entry.getKey();
                            List<StrategyControlDetailList.DailyPlan.DailyPlanDetail> value = entry.getValue();
                            List<String> times = new ArrayList<>(value.size());
                            for (StrategyControlDetailList.DailyPlan.DailyPlanDetail dailyPlanDetail : value) {
                                List<String> timesList = dailyPlanDetail.getTimes();
                                String timeStr = timesList.get(0) + "-" + timesList.get(1);
                                times.add(timeStr);
                            }
                            result.setStrategy(strategy);
                            result.setTimes(times);
                            dailyPlanGroupList.add(result);
                        }

                        for (StrategyControlDetailList.DailyPlan.DailyPlanDetail dailyPlanDetail : dailyPlanGroupList) {
                            StrategyPlanDTO strategyPlanDTO = new StrategyPlanDTO();
                            Integer strategy = dailyPlanDetail.getStrategy();
                            strategyPlanDTO.setStrategy(strategy);
                            strategyPlanDTO.setTimes(dailyPlanDetail.getTimes());
                            strategyPlanDTO.setDailyPlanId(dailyPlanId);
                            strategyPlanDTO.setCrossId(crossId);
                            strategyPlanDTOS.add(strategyPlanDTO);
                        }
                    }
                }

                List<StrategyPlanDTO> resultList = new ArrayList<>();
                List<StrategyControlDetailList.ExecutePlan> plans = list.getPlans();
                if (!CollectionUtils.isEmpty(plans) && !CollectionUtils.isEmpty(dailyPlans)) {
                    for (StrategyControlDetailList.ExecutePlan plan : plans) {
                        Integer planId = plan.getPlanId();
                        Date startTime = plan.getStartTime();
                        Date endTime = plan.getEndTime();
                        Integer status = plan.getStatus();
                        List<StrategyControlDetailList.ExecutePlan.PlanDetail> planDetails = plan.getPlanDetails();
                        if (!CollectionUtils.isEmpty(planDetails)) {
                            for (StrategyControlDetailList.ExecutePlan.PlanDetail planDetail : planDetails) {
                                for (StrategyPlanDTO strategyPlanDTO : strategyPlanDTOS) {
                                    Integer dailyPlanId = planDetail.getDailyPlanId();
                                    List<Integer> weeks = planDetail.getWeeks();
                                    if (Objects.equals(dailyPlanId, strategyPlanDTO.getDailyPlanId())) {
                                        strategyPlanDTO.setPlanId(planId);
                                        strategyPlanDTO.setStartTime(startTime);
                                        strategyPlanDTO.setEndTime(endTime);
                                        strategyPlanDTO.setWeeks(weeks);
                                        strategyPlanDTO.setStatus(status);
                                        resultList.add(strategyPlanDTO);
                                    }
                                }
                            }
                        }
                    }
                }
                List<StrategyControlDataVO> dataList = new ArrayList<>();
                for (StrategyPlanDTO strategyPlanDTO : resultList) {
                    StrategyControlDataVO dataVO = new StrategyControlDataVO();
                    dataVO.setBizId(strategyPlanDTO.getCrossId());
                    dataVO.setBizType(0);
                    dataVO.setStrategy(strategyPlanDTO.getStrategy());
                    dataVO.setScheduleStart(strategyPlanDTO.getStartTime());
                    dataVO.setScheduleEnd(strategyPlanDTO.getEndTime());
                    dataVO.setAction("insert");
                    dataVO.setStatus(strategyPlanDTO.getStatus());
                    dataVO.setFrequency(strategyPlanDTO.getFreq());
                    List<StrategyControlDataVO.TimeTable> timeTables = new ArrayList<>();
                    List<Integer> weeks = strategyPlanDTO.getWeeks();
                    for (Integer week : weeks) {
                        StrategyControlDataVO.TimeTable timeTable = new StrategyControlDataVO.TimeTable();
                        timeTable.setWeek(week);
                        List<String> times = strategyPlanDTO.getTimes();
                        timeTable.setTimeList(times.toArray(new String[times.size()]));
                        timeTables.add(timeTable);
                    }
                    dataVO.setTime(timeTables);
                    dataList.add(dataVO);
                    StrategyControlVO req = new StrategyControlVO();
                    req.setDataList(dataList);
                    return req;
                }
            }
        } catch (Exception e) {
            log.error("数据转化神思数据结构异常：{}", e);
            throw new Exception(e);
        }
        return null;
    }

    @Override
    public JsonViewObject strategyFactoryList() throws Exception {
        List<StrategyFactoryEntity> strategyFactoryEntities = strategyFactoryMapper.selectList(null);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        jsonViewObject.success(strategyFactoryEntities);
        return jsonViewObject;
    }

    @Override
    public JsonViewObject strategyFactorySave(StrategyFactoryEntity entity) throws Exception {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        try {
            if (Objects.nonNull(entity.getId())) {
                strategyFactoryMapper.updateById(entity);
            } else {
                strategyFactoryMapper.insert(entity);
            }
        } catch (Exception e) {
            log.error("策略库保存失败：{}", e);
            jsonViewObject.success("策略库保存失败");
        }
        return jsonViewObject.success("策略库保存成功");
    }

    @Override
    public JsonViewObject strategyFactoryDel(Integer id) throws Exception {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        try {
            StrategyFactoryEntity strategyFactoryEntity = new StrategyFactoryEntity();
            strategyFactoryEntity.setId(id);
            strategyFactoryMapper.deleteById(strategyFactoryEntity);
        } catch (Exception e) {
            log.error("策略库策略删除失败：{}", e);
            jsonViewObject.success("策略库策略删除失败");
        }
        return jsonViewObject.success("策略库删除成功");
    }
}
